#include <algorithm>
#include <iostream>
#include <cassert>
#include <vector>
#include <cmath>
using namespace std;
typedef long double ld;

struct vec2{
  ld x,y;
  bool operator<(vec2 const &a)const{return x!=a.x? x<a.x: y<a.y;}
  vec2 operator/(ld const v)const{return {x/v,y/v};}
  vec2 operator-(vec2 const v)const{return {x-v.x,y-v.y};}
  ld dot(vec2 const &a)const{return x*a.x+y*a.y;}
  ld atan2()const{return atan2l(y,x);}
};

vec2 perp(vec2 const &v){
  return {v.y,-v.x};
}

struct mat2{
  ld c[2][2];
  vec2 operator*(vec2 const &a)const{
    return {c[0][0]*a.x+c[1][0]*a.y,
            c[0][1]*a.x+c[1][1]*a.y};
  }
};

ld integrate(vector<pair<ld,vector<ld>>> const &fun, ld const x){
  int idex=0;
  for (int r=(1<<29); r; r>>=1){
    if (idex+r<fun.size() and fun[idex+r].first<=x) idex+=r;
  }
  auto const &f=fun[idex].second;
  return f[0]+f[1]*x+f[2]*powl(x,2);
}

vector<ld> solve(vector<vec2> const &v, ld const slice,int const want){
  /* // not needed if upvec is always {0,1}
  upvec=upvec/hypotl(upvec.x,upvec.y);
  mat2 inv={{{upvec.x,upvec.y},{-upvec.y,upvec.x}}};
  for (auto &i: v) i=inv*i;
  */

  vector<pair<ld,vector<ld>>> evt;
  for (int i=0,j=v.size()-1; i<v.size(); j=i++){
    int a=i,b=j; int mul=+1; if (v[b]<v[a]) swap(a,b), mul=-1;
    if (v[a].x==v[b].x) continue;

    // y = ay + gradient * (x-ax)
    // i(y) = (ay-gradient*ax)*x + gradient/2*x^2 - (ay-gradient*ax*ax+gradient/2*ax*ax)
    if (v[a].x+1e-9L<v[b].x){
      ld const gradient=(v[b].y-v[a].y)/(v[b].x-v[a].x);
      ld const ca=gradient/2.L;
      ld const cm=v[a].y-gradient*v[a].x;
      ld const cc=-ca*v[a].x*v[a].x-cm*v[a].x;

      evt.emplace_back(v[a].x, vector<ld>{                                 0L+cc, v[a].y-gradient*v[a].x, gradient/2.L});
      evt.emplace_back(v[b].x, vector<ld>{(v[b].x-v[a].x)*(v[a].y+v[b].y)/2.L-cc,-v[a].y+gradient*v[a].x,-gradient/2.L});
      for (auto &z: evt[evt.size()-2].second) z*=mul;
      for (auto &z: evt[evt.size()-1].second) z*=mul;
    }
  }
  sort(evt.begin(),evt.end());

  // width[x] = {distance, {c,m,a}}
  // integrate(x) = c+m*x+a*x*x
  vector<pair<ld,vector<ld>>> width={{-1e20L, {0L,0L,0L}}};
  for (int i=0,j=0; i<evt.size(); i++){
    if (i+1==evt.size() or evt[i].first+1e-8L<evt[i+1].first){
      width.emplace_back(evt[i].first, width.back().second);
      for (; j<=i; ++j){
        assert(width.back().second.size() == evt[j].second.size());
        for (int k=0; k<3; k++){
          width.back().second[k]+=evt[j].second[k];
        }
      }
    }
  }

  vector<ld> res;
  ld lp=width.begin()->first,rp=width.rbegin()->first;
  for (int i=1; i<=want; i++){
    ld need=i*slice;
    ld lef=lp,rgt=rp;
    for (int iter=200; iter--;){
      ld mid=(lef+rgt)/2.L;
      (abs(integrate(width,mid))<need?lef:rgt)=mid;
    }
    res.push_back(lef);
  }
  return res;
}

int main(){
  ld bulkhead_size; cin>>bulkhead_size;
  int n; cin>>n;
  vector<vec2> v(n); for (auto &i: v) cin>>i.x>>i.y;

  // work out how many sections we can fit in,
  // then section size is total volume divided by that
  ld area=0.L;
  for (int i=0,j=v.size()-1; i<v.size(); j=i++){
    area+=(v[i].x-v[j].x)*(v[i].y+v[j].y)/2.L;
  }
  int sections=floor(abs(area)/bulkhead_size)+.5L;
  bulkhead_size=abs(area)/sections;
  vector<ld> const res=solve(v,bulkhead_size,sections-1);
  cout.precision(16);
  cout<<res.size()+1<<endl;
  for (int i=0; i<res.size(); i++) cout<<res[i]<<endl;
}
