#include <iostream>
#include <fstream>
#include <algorithm>
#include <stack>
#include <string>
#include <vector>
#include <unordered_map>
using namespace std;

constexpr int rc_accepted = 42;
constexpr int rc_wrong_answer = 43;
constexpr int rc_judge_error = 1;

constexpr int max_instructions = 40;
constexpr int max_stack_size = 256;

int correct_answer = -1;

bool push(int *regs, int idx, stack<int> &stackref) {
  if (stackref.size() == max_stack_size) {
    cout<<"stack overflow (push)"<<endl;
    return false;
  }
  if (regs[idx] == -1) {
    cout<<"uninitialised register: "<<idx<<endl;
  }
  stackref.push(regs[idx]);
  return true;
}

bool pull(int *regs, int idx, stack<int> &stackref) {
  if (stackref.size() == 0) {
    cout<<"stack underflow (pull)"<<endl;
    return false;
  }
  regs[idx] = stackref.top(); stackref.pop();
  return true;
}

bool add(int *regs, int idx, stack<int> &stackref) {
  if (stackref.size() < 2) {
    cout<<"stack underflow (add)"<<endl;
    return false;
  }
  int a = stackref.top(); stackref.pop();
  int b = stackref.top(); stackref.pop();
  if (a == -1 || b == -1) {
    cout<<"uninitialised stack variable(s): "<<a<<" "<<b<<endl;
    stackref.push(-1);
  } else {
    stackref.push((a + b) & 255);
  }
  return true;
}

bool set_one(int *regs, int idx, stack<int> &stackref) {
  regs[idx] = 1;
  return true;
}

bool set_zero(int *regs, int idx, stack<int> &stackref) {
  regs[idx] = 0;
  return true;
}

bool display(int *regs, int idx, stack<int> &stackref) {
  if (regs[idx] == -1) {
    cout<<"uninitialised register: "<<idx<<endl;
  }

  // cout << regs[idx] << '\n';
  if (regs[idx] == correct_answer) {
    exit(rc_accepted);
    return true;
  } else {
    cout<<"Wrong answer: displayed "<<regs[idx]<<", expected "<<correct_answer<<endl;
    exit(rc_wrong_answer);
    return false;
  }
}

int main(int argc, char **argv){
  ifstream canonical_input(argv[1]);
  ifstream canonical_output(argv[2]); // ignored
  auto &team_output = cin;

  // Save the input (which should be the output of our brave new team's solution)
  canonical_input>>correct_answer;

  unordered_map<string, pair<size_t, decltype(&display)>> function_map {
    {"PL", {1, &pull}},
    {"PH", {1, &push}},
    {"AD", {0, &add}},
    {"ST", {1, &set_one}},
    {"ZE", {1, &set_zero}},
    {"DI", {1, &display}}
  };
  unordered_map <string, int> reg_map {{"A",0}, {"X",1}, {"Y",2}};
  stack<int> stck;

  bool done = false;
  int regs[4] {-1, -1, -1, -1};
  int idx = 0;
  int instr_count = 0;

  while (instr_count < max_instructions && cin.good()) {
    string instruction;
    getline(cin, instruction);

    // skip empty_lines.
    if (instruction.length() == 0) {
      continue;
    }

    // check for valid instructions.
    if (instruction.length() != 2 && instruction.length() != 4) {
      cout<<"wrong length for instruction: "<<instruction<<endl;
      return rc_wrong_answer;
    }

    string const opcode=instruction.substr(0,2);
    if (not function_map.count(opcode)) {
      cout<<"function not found: "<<opcode<<endl;
      return rc_wrong_answer;
    }

    vector<int> args;
    for (size_t i=3; i<instruction.length(); i+=2) {
      // check registers are correct.
      const string arg = instruction.substr(i,1);
      auto it = reg_map.find(arg);
      if (it == reg_map.end()) {
        cout<<"no such register: "<<arg<<endl;
        return rc_wrong_answer;
      }
      args.push_back(reg_map[arg]);
    }
    // check for the right number of arguments.
    if (args.size() != function_map[opcode].first){
      cout<<"wrong number of arguments for "<<opcode<<"."
          <<" got:"<<args.size()
          <<" expected:"<<function_map[opcode].first<<endl;
      return rc_wrong_answer;
    }

    // run the instruction.
    if(!function_map[opcode].second(regs, args.empty() ? 3: args[0], stck)) {
      cout<<"asm call failed: "<<instruction<<endl;
      return rc_wrong_answer;
    }
    instr_count++;
  }
  return rc_wrong_answer;
}
