#include <iostream>
#include <cassert>
#include <sstream>
#include <cstring>
#include <vector>
#include <cmath>
#include <map>
using namespace std;
typedef long double ld;

char cell[41][41];
ld val[41][41];

ld sqist(ld const a,ld const b,ld const c){
  return powl(a,2)+powl(b,2)+powl(c,2);
}

constexpr int v_s=0;
constexpr int v_t=1;
constexpr int lots=10000000;
constexpr int v_max=2+40*40;

constexpr int cost_dark=11;
constexpr int cost_light=43;

map<int,int> edge[v_max];
static int dist[v_max];

bool bfs(int s,int t){
  memset(dist,-1,sizeof dist);
  vector<int> q={s}; dist[s]=0;
  for (int i=0; i<q.size(); i++)
    if (q[i]==t)
      return true;
    else
      for (auto y: edge[q[i]])
        if (y.second>0 and !~dist[y.first])
          dist[y.first]=dist[q[i]]+1,
          q.push_back(y.first);
  return false;
}

int dfs(int s,int t,int limit=lots){
  if (s==t) return limit;
  int res=0;
  for (auto &y: edge[s]){
    if (limit>0 and y.second>0 and dist[y.first]==dist[s]+1){
      int r=dfs(y.first,t,min(limit,y.second));
      if (r<=min(limit,y.second)) dist[y.first]=-1;
      limit-=r, res+=r;
      y.second-=r, edge[y.first][s]+=r;
    }
  }
  if (limit) dist[s]=-1;
  return res;
}

int main(){
  int min_brightness; cin>>min_brightness;
  int ceiling_height; cin>>ceiling_height;
  int r,c; cin>>r>>c;

  for (int i=0; i<r; i++){
    cin>>cell[i];
    assert(strnlen(cell[i],sizeof cell[i])==c);
    for (int j=0; j<c; j++){
      cell[i][j]-='0';
    }
  }

  for (int i=0; i<r; i++){
    for (int j=0; j<c; j++){
      for (int a=0; a<r; a++){
        for (int b=0; b<c; b++){
          val[i][j]+=cell[a][b]/sqist(a-i,b-j,ceiling_height);
        }
      }
      val[i][j]=(val[i][j]+1e-9L>=min_brightness);
    }
  }

  for (int i=0; i<r; i++)
    for (int j=0; j<c; j++){
      if (val[i][j] and (i==0 or j==0 or i+1==r or j+1==c)){
        // must keep this vertex
        edge[v_s][i*c+j+2]=r*c*4*(cost_light+cost_dark);
      }
      if (not val[i][j]){
        // need to cut this vertex somehow
        edge[i*c+j+2][v_t]=r*c*4*(cost_light+cost_dark);
      }
      for (int dx=1,dy=0,z=4; z--; swap(dx,dy*=-1))
        if (i+dx>=0 and j+dy>=0 and i+dx<r and j+dy<c)
          edge[i*c+j+2][(i+dx)*c+(j+dy)+2]=(val[i][j] and val[i+dx][j+dy]? cost_light: cost_dark);
    }

  int res=0;
  while (bfs(v_s,v_t)) res+=dfs(v_s,v_t);
  cout<<res<<endl;

  if (true and res) /* print the actual solution */ {
    vector<string> s;
    for (int i=0; i<r; i++){
      {
        stringstream ss;
        for (int j=0; j<c; j++){
          ss<<val[i][j];
          if (j+1<c) ss<<((!~dist[i*c+j+2])!=(!~dist[i*c+j+3])? '|': ' ');
        }
        s.push_back(ss.str());
      }
      {
        stringstream ss;
        if (i+1<r){
          for (int j=0; j<c; j++){
            ss<<((!~dist[i*c+j+2])!=(!~dist[(i+1)*c+j+2])? '-': ' ')<<' ';
          }
        }
        s.push_back(ss.str());
      }
    }
    for (int i=0; i<s.size(); i++)
      for (int j=0; j+2<s[i].size(); j++)
        if (s[i][j]=='-' and s[i][j+2]=='-') s[i][j+1]='-';
    for (int j=0; j<s[0].size(); j++)
      for (int i=0; i+2<s.size(); i++)
        if (s[i][j]=='|' and s[i+2][j]=='|') s[i+1][j]='|';
    for (auto i: s) cerr<<i<<endl;
  }
}
