#!/usr/bin/env perl

use strict;
use warnings;
use List::MoreUtils qw(uniq);

my @letters = ('a'..'z');
my @chars = (@letters,(' ') x 20);

my $e = 10;
my $p = 10;

# E lines, each no more than 100 letters in length. Each a list of space separated word endings at
# least one letter in length.
my %suffixes = ();
my @wordses = ();
for my $i (1 .. $e) {
    my $line = join('', map {$chars[int rand @chars]} (1..100));
    $line =~ s/^\s+|\s+$//g;
    $line =~ s/\s+/ /g;
    my @words = uniq sort (split / /, $line);
    for my $j (@words) {
        $suffixes{$j} = 1;
    }
    push @wordses, "@words";
}
my @suffixes = keys %suffixes;

# One line containing the single common word, at least one letter long, no more than 20 long.
my $common = $suffixes[int rand @suffixes];
if (length($common) > 20) {
    $common = substr($common, length($common)-20);
}
print "$common\n";

# One line containing an integer $E$ ($0 < E \le 10$), the number of lists of word endings that
# sound the same.
print "$e\n";
for my $word (@wordses) {
    print "$word\n";
}

# One line containing an integer $P$ ($0 < P \le 10$), the number of phrases to test.
print "$p\n";

# P lines, each containing a phrase $p_i$ of two or three words that might rhyme with the common
# word.
for my $i (1 .. $p) {
    my $line = "";
    for my $q (1 .. 3) {
        my $add = $suffixes[int rand @suffixes];
        $add = join('', map {$letters[int rand @letters]} (1..1+(int rand 4))) . $add;
        $add = substr($add, int rand (length($add) - 1));
        if (length $line) {
            $add = " $add";
        }
        next if length($line) + length($add) > 100;
        $line .= $add;
    }
    print "$line\n";
}
