// JWG first attempt at a shortest path

#include <iostream>
#include <vector>
#include <array>
#include <algorithm>
using namespace std;

// Create a structure that has the type, and an array of links to other cells.

#define MAX_SIZE 400*400

typedef struct Cell {
	char type;
	bool finish = false;
	int min_distance = MAX_SIZE + 1;
	vector<Cell *> adj;
	int track;  
} Cell;

bool compareCells(Cell *a, Cell *b) {
	return a->min_distance < b->min_distance;
};

int shortest_path(vector<Cell *> unvisited) {
	Cell *current;

	while(unvisited.size() > 0) {
		current = unvisited[0];
		if (current->finish == true) {
			break;
		}

		for(unsigned i = 0; i < current->adj.size(); i++) {
			auto neighbour = current->adj[i];
			int length = neighbour->type == 'D' ? 0 : 1;
			if (neighbour->min_distance > current->min_distance + length) {
				neighbour->min_distance = current->min_distance + length;
			}
		}

		// remove the one we've just done.
		unvisited.erase(unvisited.begin());

		// Sort the unvisited cells by min_distance.
		std::sort(unvisited.begin(), unvisited.end(), compareCells);
	}
	return current->min_distance;
}; 



int main(){
	// Read in the data.
	int ignore;
	int w,l;
	ignore = scanf(" %d %d",&l,&w);

	vector<Cell> map(w * l);
	array<vector<Cell *>, MAX_SIZE + 1> adj;
	char line[500];

	for(int i = 0; i < l; i++) {
		ignore = scanf(" %s",line);
		for(int j = 0; j < w; j++) {
			map[j + (i * w)].type = line[j];
			map[j + (i * w)].track = j + (i * w);
		}
	}

	int x, y;
	ignore = scanf(" %d %d",&y,&x);
	x--; y--;
	map[x + (y * w)].finish = true;
	
	// Got the data, now build up the graph.
	for(int j = 0; j < (l * w); j++ ) {
		if (map[j].type == '#') {
			// Not interested in what is next to walls.
			continue;
		}
		// If we're not on the first row, look up a row.
		if ( j > (w - 1) ) {
			if (map[j - w].type == 'c' || map[j - w].type == 'D') {
				map[j].adj.push_back(&(map[j - w]));
			}
		}
		// If we're not on the last row, look down a row.
		if ( j < (l * w) - (w - 1)) {
			if (map[j + w].type == 'c' || map[j + w].type == 'D') {
				map[j].adj.push_back(&(map[j + w]));
			}	
		}
		// If we're not on the first column, look back a column.
		if ( j % w > 0 ) {
			if (map[j - 1].type == 'c' || map[j - 1].type == 'D') {
				map[j].adj.push_back(&(map[j - 1]));
			}
		}
		// If we're not on the last column, look forward a column.
		if ( (j % w) != (w - 1)) {
			if (map[j + 1].type == 'c' || map[j + 1].type == 'D') {
				map[j].adj.push_back(&(map[j + 1]));
			}
		}
	}
	
	// Find each door on the outside.
	vector<int> doors;

	for(int i = 0; i < l * w; i++) {
		if (((i < w) || (i > (l*w) - (w + 1)) || (i % w == 0) || (i % w == w - 1))
			&& map[i].type == 'D') {
			doors.push_back(i);
		}
	}

	int shortest = 200000,
		door_shortest;
	Cell *start;
	vector<Cell *> unvisited;

	for(unsigned it = 0; it < doors.size(); it++) {
	    // Make a list of unvisited nodes, initialise each node.
		for(int i = 0; i < w * l; i++) {
			if (i == doors[it]) {
				map[i].min_distance = 0;
				start = &(map[i]);

			} else if (map[i].type != '#') {
				map[i].min_distance = 200001;
				unvisited.push_back(&(map[i]));
			}
		}
		unvisited.insert(unvisited.begin(), start);
		door_shortest = shortest_path(unvisited);
		if ( door_shortest < shortest) {
			shortest = door_shortest;
		}
		unvisited.clear();
	}
	cout << shortest << "\n";
};
