#!/bin/bash

shopt -s nullglob

if [ "$#" -ne 0 ]
  then
    problems="$@"
  else
    problems="$(for i in ./problems/*/ ; do basename "$i" ; done)"
  fi

for problem in $problems
  do
    checker="diff -wB"
    if [ -e ./problems/"$problem"/data/checker ]
      then
        checker=./problems/"$problem"/data/checker
      fi

    if [[ ! -z $problem ]] && [ -e ./problems/"$problem" ]
      then
        printf 'Checking problem "%s"\n' "$problem"
      else
        printf 'Error: no problem exists named "%s"\n' "$problem" >&2
        exit 1
      fi

    ctd_ok=0
    ctd_total=0
    type checktestdata >/dev/null 2>&1 && \
      for validator in ./problems/"$problem"/input_format_validators/*.ctd
        do
          for type in "sample" "secret"
            do
              for tf in ./problems/"$problem"/data/"$type"/*.in
                do
                  op=$( checktestdata "$validator" "$tf" )
                  [ $? == 0 ] && ((ctd_ok++))
                  ((ctd_total++))
                done
            done
          printf '  Validated %d/%d tests\n' "$ctd_ok" "$ctd_total"
        done \
      || echo '  Skipping checktestdata'

    for codefile in ./problems/"$problem"/submissions/accepted/*.*
      do
        executable=./problems/"$problem"/a.out
        friendly="`basename "$codefile"`"
        ext="${codefile##*.}"

        tests=0; ac=0;

        if [ "$ext" = "py" -o "$ext" = "py2" -o "$ext" = "py3" ]
          then
            cp "$codefile" "$executable"
        elif [ "$ext" = "java" ]
          then
            javac "$codefile"
            if [ $? -ne 0 ]
              then
                printf 'Failed to compile %s\n' "$friendly" >&2
                continue
            else
                echo "#!/bin/bash" > "$executable"
                echo "cd $(dirname "$codefile")" >> "$executable"
                echo "java $(basename "$friendly" .java)" >> "$executable"
              fi
        elif [ "$ext" = "class" ]
          then
            continue
        elif [ "$ext" = "cc" -o "$ext" = "$c" -o "$ext" = "cxx" -o "$ext" = "cpp" ]
          then
            g++ -std=c++14 -O2 -o "$executable" "$codefile"
            if [ $? -ne 0 ]
              then
                printf 'Failed to compile %s\n' "$friendly" >&2
                continue
              fi
        elif [ "$ext" = "c" ]
          then
            gcc -std=c11 -O2 -o "$executable" "$codefile"
            if [ $? -ne 0 ]
              then
                printf 'Failed to compile %s\n' "$friendly" >&2
                continue
              fi
        else
            printf '  skipping "%s"\n' "$friendly"
            continue
          fi

        chmod "+x" "$executable"

        echo -n '  '
        for type in "sample" "secret"
          do
            for tf in ./problems/"$problem"/data/"$type"/*.in
              do
                tn="`basename "$tf" .in`"
                answer="problems/"$problem"/data/"$type"/"$tn".ans"
                if [ -f "$answer" ]
                  then
                    checker_output="`$checker <("$executable" < "$tf" 2>/dev/null) "$answer"`"
                    if [ -z "$checker_output" ] || [ "$checker_output" = "OK" ]
                      then
                        echo -n '+' && ((ac++))
                      else
                        printf '\nTest %s %s:\n%s\n' "$type" "$tn" "$checker_output" >&2
                        echo -n '-'
                      fi
                    ((tests++))
                  else
                    "$executable" <"$tf" >"$answer.maybe" 2>/dev/null
                    echo -n '?'
                  fi
              done
          done
        printf '%3d/%d %s\n' "$ac" "$tests" "$friendly"

        rm $executable
        for class in $(dirname "$codefile")/*.class
          do
            rm "$class"
          done

        if [ $tests == 0 ]
          then
            echo "No tests"
            break
          fi
      done
    echo
  done

exit 0
